# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region as ScwRegion,
    Zone as ScwZone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class AclActionRedirectRedirectType(str, Enum, metaclass=StrEnumMeta):
    LOCATION = "location"
    SCHEME = "scheme"

    def __str__(self) -> str:
        return str(self.value)


class AclActionType(str, Enum, metaclass=StrEnumMeta):
    ALLOW = "allow"
    DENY = "deny"
    REDIRECT = "redirect"

    def __str__(self) -> str:
        return str(self.value)


class AclHttpFilter(str, Enum, metaclass=StrEnumMeta):
    ACL_HTTP_FILTER_NONE = "acl_http_filter_none"
    PATH_BEGIN = "path_begin"
    PATH_END = "path_end"
    REGEX = "regex"
    HTTP_HEADER_MATCH = "http_header_match"

    def __str__(self) -> str:
        return str(self.value)


class BackendServerStatsHealthCheckStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    NEUTRAL = "neutral"
    FAILED = "failed"
    PASSED = "passed"
    CONDPASS = "condpass"

    def __str__(self) -> str:
        return str(self.value)


class BackendServerStatsServerState(str, Enum, metaclass=StrEnumMeta):
    STOPPED = "stopped"
    STARTING = "starting"
    RUNNING = "running"
    STOPPING = "stopping"

    def __str__(self) -> str:
        return str(self.value)


class CertificateStatus(str, Enum, metaclass=StrEnumMeta):
    PENDING = "pending"
    READY = "ready"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)


class CertificateType(str, Enum, metaclass=StrEnumMeta):
    LETSENCRYT = "letsencryt"
    CUSTOM = "custom"

    def __str__(self) -> str:
        return str(self.value)


class ForwardPortAlgorithm(str, Enum, metaclass=StrEnumMeta):
    ROUNDROBIN = "roundrobin"
    LEASTCONN = "leastconn"
    FIRST = "first"

    def __str__(self) -> str:
        return str(self.value)


class InstanceStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    PENDING = "pending"
    STOPPED = "stopped"
    ERROR = "error"
    LOCKED = "locked"
    MIGRATING = "migrating"

    def __str__(self) -> str:
        return str(self.value)


class LbStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    PENDING = "pending"
    STOPPED = "stopped"
    ERROR = "error"
    LOCKED = "locked"
    MIGRATING = "migrating"
    TO_CREATE = "to_create"
    CREATING = "creating"
    TO_DELETE = "to_delete"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class LbTypeStock(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    LOW_STOCK = "low_stock"
    OUT_OF_STOCK = "out_of_stock"
    AVAILABLE = "available"

    def __str__(self) -> str:
        return str(self.value)


class ListAclRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListBackendsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListCertificatesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListFrontendsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListIpsRequestIpType(str, Enum, metaclass=StrEnumMeta):
    ALL = "all"
    IPV4 = "ipv4"
    IPV6 = "ipv6"

    def __str__(self) -> str:
        return str(self.value)


class ListLbsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPrivateNetworksRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRoutesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListSubscriberRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class OnMarkedDownAction(str, Enum, metaclass=StrEnumMeta):
    ON_MARKED_DOWN_ACTION_NONE = "on_marked_down_action_none"
    SHUTDOWN_SESSIONS = "shutdown_sessions"

    def __str__(self) -> str:
        return str(self.value)


class PrivateNetworkStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    PENDING = "pending"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)


class Protocol(str, Enum, metaclass=StrEnumMeta):
    TCP = "tcp"
    HTTP = "http"

    def __str__(self) -> str:
        return str(self.value)


class ProxyProtocol(str, Enum, metaclass=StrEnumMeta):
    PROXY_PROTOCOL_UNKNOWN = "proxy_protocol_unknown"
    PROXY_PROTOCOL_NONE = "proxy_protocol_none"
    PROXY_PROTOCOL_V1 = "proxy_protocol_v1"
    PROXY_PROTOCOL_V2 = "proxy_protocol_v2"
    PROXY_PROTOCOL_V2_SSL = "proxy_protocol_v2_ssl"
    PROXY_PROTOCOL_V2_SSL_CN = "proxy_protocol_v2_ssl_cn"

    def __str__(self) -> str:
        return str(self.value)


class SSLCompatibilityLevel(str, Enum, metaclass=StrEnumMeta):
    SSL_COMPATIBILITY_LEVEL_UNKNOWN = "ssl_compatibility_level_unknown"
    SSL_COMPATIBILITY_LEVEL_INTERMEDIATE = "ssl_compatibility_level_intermediate"
    SSL_COMPATIBILITY_LEVEL_MODERN = "ssl_compatibility_level_modern"
    SSL_COMPATIBILITY_LEVEL_OLD = "ssl_compatibility_level_old"

    def __str__(self) -> str:
        return str(self.value)


class StickySessionsType(str, Enum, metaclass=StrEnumMeta):
    NONE = "none"
    COOKIE = "cookie"
    TABLE = "table"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class SubscriberEmailConfig:
    email: str
    """
    Email address to send alerts to.
    """


@dataclass
class SubscriberWebhookConfig:
    """
    Webhook alert of subscriber.
    """

    uri: str
    """
    URI to receive POST requests.
    """


@dataclass
class HealthCheckHttpConfig:
    uri: str
    """
    The HTTP path to use when performing a health check on backend servers.
    """

    method: str
    """
    The HTTP method used when performing a health check on backend servers.
    """

    host_header: str
    """
    The HTTP host header used when performing a health check on backend servers.
    """

    code: Optional[int]
    """
    The HTTP response code that should be returned for a health check to be considered successful.
    """


@dataclass
class HealthCheckHttpsConfig:
    uri: str
    """
    The HTTP path to use when performing a health check on backend servers.
    """

    method: str
    """
    The HTTP method used when performing a health check on backend servers.
    """

    host_header: str
    """
    The HTTP host header used when performing a health check on backend servers.
    """

    sni: str
    """
    The SNI value used when performing a health check on backend servers over SSL.
    """

    code: Optional[int]
    """
    The HTTP response code that should be returned for a health check to be considered successful.
    """


@dataclass
class HealthCheckLdapConfig:
    pass


@dataclass
class HealthCheckMysqlConfig:
    user: str
    """
    MySQL user to use for the health check.
    """


@dataclass
class HealthCheckPgsqlConfig:
    user: str
    """
    PostgreSQL user to use for the health check.
    """


@dataclass
class HealthCheckRedisConfig:
    pass


@dataclass
class HealthCheckTcpConfig:
    pass


@dataclass
class Instance:
    id: str
    """
    Underlying Instance ID.
    """

    status: InstanceStatus
    """
    Instance status.
    """

    ip_address: str
    """
    Instance IP address.
    """

    zone: ScwZone
    """
    The zone the Instance is in.
    """

    created_at: Optional[datetime]
    """
    Date on which the Instance was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the Instance was last updated.
    """

    region: Optional[ScwRegion]
    """
    The region the Instance is in.
    """


@dataclass
class Ip:
    id: str
    """
    IP address ID.
    """

    ip_address: str
    """
    IP address.
    """

    organization_id: str
    """
    Organization ID of the Scaleway Organization the IP address is in.
    """

    project_id: str
    """
    Project ID of the Scaleway Project the IP address is in.
    """

    reverse: str
    """
    Reverse DNS (domain name) of the IP address.
    """

    tags: List[str]
    """
    IP tags.
    """

    zone: ScwZone
    """
    The zone the IP address is in.
    """

    lb_id: Optional[str]
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    The region the IP address is in.
    """


@dataclass
class Subscriber:
    """
    Subscriber.
    """

    id: str
    """
    Subscriber ID.
    """

    name: str
    """
    Subscriber name.
    """

    email_config: Optional[SubscriberEmailConfig]

    webhook_config: Optional[SubscriberWebhookConfig]


@dataclass
class HealthCheck:
    port: int
    """
    Port to use for the backend server health check.
    """

    check_max_retries: int
    """
    Number of consecutive unsuccessful health checks after which the server will be considered dead.
    """

    check_send_proxy: bool
    """
    Defines whether proxy protocol should be activated for the health check.
    """

    check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks.
    """

    check_timeout: Optional[str]
    """
    Maximum time a backend server has to reply to the health check.
    """

    transient_check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks when a backend server is in a transient state (going UP or DOWN).
    """

    tcp_config: Optional[HealthCheckTcpConfig]

    mysql_config: Optional[HealthCheckMysqlConfig]

    pgsql_config: Optional[HealthCheckPgsqlConfig]

    ldap_config: Optional[HealthCheckLdapConfig]

    redis_config: Optional[HealthCheckRedisConfig]

    http_config: Optional[HealthCheckHttpConfig]

    https_config: Optional[HealthCheckHttpsConfig]


@dataclass
class Lb:
    id: str
    """
    Underlying Instance ID.
    """

    name: str
    """
    Load Balancer name.
    """

    description: str
    """
    Load Balancer description.
    """

    status: LbStatus
    """
    Load Balancer status.
    """

    instances: List[Instance]
    """
    List of underlying Instances.
    """

    organization_id: str
    """
    Scaleway Organization ID.
    """

    project_id: str
    """
    Scaleway Project ID.
    """

    ip: List[Ip]
    """
    List of IP addresses attached to the Load Balancer.
    """

    tags: List[str]
    """
    Load Balancer tags.
    """

    frontend_count: int
    """
    Number of frontends the Load Balancer has.
    """

    backend_count: int
    """
    Number of backends the Load Balancer has.
    """

    type_: str
    """
    Load Balancer offer type.
    """

    ssl_compatibility_level: SSLCompatibilityLevel
    """
    Determines the minimal SSL version which needs to be supported on client side.
    """

    private_network_count: int
    """
    Number of Private Networks attached to the Load Balancer.
    """

    route_count: int
    """
    Number of routes configured on the Load Balancer.
    """

    zone: ScwZone
    """
    The zone the Load Balancer is in.
    """

    subscriber: Optional[Subscriber]
    """
    Subscriber information.
    """

    created_at: Optional[datetime]
    """
    Date on which the Load Balancer was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the Load Balancer was last updated.
    """

    region: Optional[ScwRegion]
    """
    The region the Load Balancer is in.
    """


@dataclass
class AclActionRedirect:
    type_: AclActionRedirectRedirectType
    """
    Redirect type.
    """

    target: str
    """
    Redirect target. For a location redirect, you can use a URL e.g. `https://scaleway.com`. Using a scheme name (e.g. `https`, `http`, `ftp`, `git`) will replace the request's original scheme. This can be useful to implement HTTP to HTTPS redirects. Valid placeholders that can be used in a `location` redirect to preserve parts of the original request in the redirection URL are \{\{host\}\}, \{\{query\}\}, \{\{path\}\} and \{\{scheme\}\}.
    """

    code: Optional[int]
    """
    HTTP redirect code to use. Valid values are 301, 302, 303, 307 and 308. Default value is 302.
    """


@dataclass
class Backend:
    id: str
    """
    Backend ID.
    """

    name: str
    """
    Name of the backend.
    """

    forward_protocol: Protocol
    """
    Protocol used by the backend when forwarding traffic to backend servers.
    """

    forward_port: int
    """
    Port used by the backend when forwarding traffic to backend servers.
    """

    forward_port_algorithm: ForwardPortAlgorithm
    """
    Load balancing algorithm to use when determining which backend server to forward new traffic to.
    """

    sticky_sessions: StickySessionsType
    """
    Defines whether sticky sessions (binding a particular session to a particular backend server) are activated and the method to use if so. None disables sticky sessions. Cookie-based uses an HTTP cookie to stick a session to a backend server. Table-based uses the source (client) IP address to stick a session to a backend server.
    """

    sticky_sessions_cookie_name: str
    """
    Cookie name for cookie-based sticky sessions.
    """

    pool: List[str]
    """
    List of IP addresses of backend servers attached to this backend.
    """

    on_marked_down_action: OnMarkedDownAction
    """
    Action to take when a backend server is marked as down.
    """

    proxy_protocol: ProxyProtocol
    """
    Protocol to use between the Load Balancer and backend servers. Allows the backend servers to be informed of the client's real IP address. The PROXY protocol must be supported by the backend servers' software.
    """

    health_check: Optional[HealthCheck]
    """
    Object defining the health check to be carried out by the backend when checking the status and health of backend servers.
    """

    lb: Optional[Lb]
    """
    Load Balancer the backend is attached to.
    """

    send_proxy_v2: Optional[bool]
    """
    Deprecated in favor of proxy_protocol field.
    """

    timeout_server: Optional[str]
    """
    Maximum allowed time for a backend server to process a request.
    """

    timeout_connect: Optional[str]
    """
    Maximum allowed time for establishing a connection to a backend server.
    """

    timeout_tunnel: Optional[str]
    """
    Maximum allowed tunnel inactivity time after Websocket is established (takes precedence over client and server timeout).
    """

    created_at: Optional[datetime]
    """
    Date at which the backend was created.
    """

    updated_at: Optional[datetime]
    """
    Date at which the backend was updated.
    """

    failover_host: Optional[str]
    """
    Scaleway Object Storage bucket website to be served as failover if all backend servers are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
    """

    ssl_bridging: Optional[bool]
    """
    Defines whether to enable SSL bridging between the Load Balancer and backend servers.
    """

    ignore_ssl_server_verify: Optional[bool]
    """
    Defines whether the server certificate verification should be ignored.
    """

    redispatch_attempt_count: Optional[int]
    """
    Whether to use another backend server on each attempt.
    """

    max_retries: Optional[int]
    """
    Number of retries when a backend server connection failed.
    """

    max_connections: Optional[int]
    """
    Maximum number of connections allowed per backend server.
    """

    timeout_queue: Optional[str]
    """
    Maximum time for a request to be left pending in queue when `max_connections` is reached.
    """


@dataclass
class Certificate:
    type_: CertificateType
    """
    Certificate type (Let's Encrypt or custom).
    """

    id: str
    """
    Certificate ID.
    """

    common_name: str
    """
    Main domain name of certificate.
    """

    subject_alternative_name: List[str]
    """
    Alternative domain names.
    """

    fingerprint: str
    """
    Identifier (SHA-1) of the certificate.
    """

    status: CertificateStatus
    """
    Certificate status.
    """

    not_valid_before: Optional[datetime]
    """
    Lower validity bound.
    """

    not_valid_after: Optional[datetime]
    """
    Upper validity bound.
    """

    lb: Optional[Lb]
    """
    Load Balancer object the certificate is attached to.
    """

    name: str
    """
    Certificate name.
    """

    created_at: Optional[datetime]
    """
    Date on which the certificate was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the certificate was last updated.
    """

    status_details: Optional[str]
    """
    Additional information about the certificate status (useful in case of certificate generation failure, for example).
    """


@dataclass
class AclAction:
    type_: AclActionType
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    redirect: Optional[AclActionRedirect]
    """
    Redirection parameters when using an ACL with a `redirect` action.
    """


@dataclass
class AclMatch:
    ip_subnet: List[str]
    """
    List of IPs or CIDR v4/v6 addresses to filter for from the client side.
    """

    http_filter: AclHttpFilter
    """
    Type of HTTP filter to match. Extracts the request's URL path, which starts at the first slash and ends before the question mark (without the host part). Defines where to filter for the http_filter_value. Only supported for HTTP backends.
    """

    http_filter_value: List[str]
    """
    List of values to filter for.
    """

    invert: bool
    """
    Defines whether to invert the match condition. If set to `true`, the ACL carries out its action when the condition DOES NOT match.
    """

    http_filter_option: Optional[str]
    """
    Name of the HTTP header to filter on if `http_header_match` was selected in `http_filter`.
    """


@dataclass
class Frontend:
    id: str
    """
    Frontend ID.
    """

    name: str
    """
    Name of the frontend.
    """

    inbound_port: int
    """
    Port the frontend listens on.
    """

    certificate_ids: List[str]
    """
    List of SSL/TLS certificate IDs to bind to the frontend.
    """

    enable_http3: bool
    """
    Defines whether to enable HTTP/3 protocol on the frontend.
    """

    backend: Optional[Backend]
    """
    Backend object the frontend is attached to.
    """

    lb: Optional[Lb]
    """
    Load Balancer object the frontend is attached to.
    """

    timeout_client: Optional[str]
    """
    Maximum allowed inactivity time on the client side.
    """

    certificate: Optional[Certificate]
    """
    Certificate, deprecated in favor of certificate_ids array.
    """

    created_at: Optional[datetime]
    """
    Date on which the frontend was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the frontend was last updated.
    """

    connection_rate_limit: Optional[int]
    """
    Rate limit for new connections established on this frontend. Use 0 value to disable, else value is connections per second.
    """


@dataclass
class PrivateNetworkDHCPConfig:
    ip_id: Optional[str]


@dataclass
class PrivateNetworkIpamConfig:
    pass


@dataclass
class PrivateNetworkStaticConfig:
    ip_address: Optional[List[str]]
    """
    Array of a local IP address for the Load Balancer on this Private Network.
    """


@dataclass
class RouteMatch:
    match_subdomains: bool
    """
    If true, all subdomains will match.
    """

    sni: Optional[str]

    host_header: Optional[str]


@dataclass
class CreateCertificateRequestCustomCertificate:
    certificate_chain: str
    """
    Full PEM-formatted certificate, consisting of the entire certificate chain including public key, private key, and (optionally) Certificate Authorities.
    """


@dataclass
class CreateCertificateRequestLetsencryptConfig:
    common_name: str
    """
    Main domain name of certificate (this domain must exist and resolve to your Load Balancer IP address).
    """

    subject_alternative_name: List[str]
    """
    Alternative domain names (all domain names must exist and resolve to your Load Balancer IP address).
    """


@dataclass
class BackendServerStats:
    instance_id: str
    """
    ID of your Load Balancer's underlying Instance.
    """

    backend_id: str
    """
    Backend ID.
    """

    ip: str
    """
    IPv4 or IPv6 address of the backend server.
    """

    server_state: BackendServerStatsServerState
    """
    Server operational state (stopped/starting/running/stopping).
    """

    last_health_check_status: BackendServerStatsHealthCheckStatus
    """
    Last health check status (unknown/neutral/failed/passed/condpass).
    """

    server_state_changed_at: Optional[datetime]
    """
    Time since last operational change.
    """


@dataclass
class Acl:
    id: str
    """
    ACL ID.
    """

    name: str
    """
    ACL name.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    description: str
    """
    ACL description.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` & `http_filter_value` are required.
    """

    action: Optional[AclAction]
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    frontend: Optional[Frontend]
    """
    ACL is attached to this frontend object.
    """

    created_at: Optional[datetime]
    """
    Date on which the ACL was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the ACL was last updated.
    """


@dataclass
class PrivateNetwork:
    ipam_ids: List[str]
    """
    IPAM IDs of the booked IP addresses.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    status: PrivateNetworkStatus
    """
    Status of Private Network connection.
    """

    lb: Optional[Lb]
    """
    Load Balancer object which is attached to the Private Network.
    """

    created_at: Optional[datetime]
    """
    Date on which the Private Network was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the PN was last updated.
    """

    static_config: Optional[PrivateNetworkStaticConfig]

    dhcp_config: Optional[PrivateNetworkDHCPConfig]

    ipam_config: Optional[PrivateNetworkIpamConfig]


@dataclass
class LbType:
    name: str
    """
    Load Balancer commercial offer type name.
    """

    stock_status: LbTypeStock
    """
    Current stock status for a given Load Balancer type.
    """

    description: str
    """
    Load Balancer commercial offer type description.
    """

    zone: ScwZone
    """
    The zone the Load Balancer stock is in.
    """

    region: Optional[ScwRegion]
    """
    The region the Load Balancer stock is in.
    """


@dataclass
class Route:
    id: str
    """
    Route ID.
    """

    frontend_id: str
    """
    ID of the source frontend.
    """

    backend_id: str
    """
    ID of the target backend.
    """

    match: Optional[RouteMatch]
    """
    Object defining the match condition for a route to be applied. If an incoming client session matches the specified condition (i.e. it has a matching SNI value or HTTP Host header value), it will be passed to the target backend.
    """

    created_at: Optional[datetime]
    """
    Date on which the route was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the route was last updated.
    """


@dataclass
class AclSpec:
    name: str
    """
    ACL name.
    """

    action: AclAction
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    description: str
    """
    ACL description.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` and `http_filter_value` are required.
    """


@dataclass
class AddBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses to add to backend servers.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class AttachPrivateNetworkRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    ipam_ids: Optional[List[str]]
    """
    IPAM ID of a pre-reserved IP address to assign to the Load Balancer on this Private Network. In the future, it will be possible to specify multiple IPs in this field (IPv4 and IPv6), for now only one ID of an IPv4 address is expected. When null, a new private IP address is created for the Load Balancer on this Private Network.
    """

    static_config: Optional[PrivateNetworkStaticConfig]

    dhcp_config: Optional[PrivateNetworkDHCPConfig]

    ipam_config: Optional[PrivateNetworkIpamConfig]


@dataclass
class CreateAclRequest:
    """
    Add an ACL to a Load Balancer frontend.
    """

    frontend_id: str
    """
    Frontend ID to attach the ACL to.
    """

    action: AclAction
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    description: str
    """
    ACL description.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    ACL name.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` & `http_filter_value` are required.
    """


@dataclass
class CreateBackendRequest:
    forward_protocol: Protocol
    """
    Protocol to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port: int
    """
    Port to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port_algorithm: ForwardPortAlgorithm
    """
    Load balancing algorithm to be used when determining which backend server to forward new traffic to.
    """

    sticky_sessions: StickySessionsType
    """
    Defines whether to activate sticky sessions (binding a particular session to a particular backend server) and the method to use if so. None disables sticky sessions. Cookie-based uses an HTTP cookie TO stick a session to a backend server. Table-based uses the source (client) IP address to stick a session to a backend server.
    """

    sticky_sessions_cookie_name: str
    """
    Cookie name for cookie-based sticky sessions.
    """

    lb_id: str
    """
    Load Balancer ID.
    """

    health_check: HealthCheck
    """
    Object defining the health check to be carried out by the backend when checking the status and health of backend servers.
    """

    server_ip: List[str]
    """
    List of backend server IP addresses (IPv4 or IPv6) the backend should forward traffic to.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the backend.
    """

    send_proxy_v2: Optional[bool]
    """
    Deprecated in favor of proxy_protocol field.
    """

    timeout_server: Optional[str]
    """
    Maximum allowed time for a backend server to process a request.
    """

    timeout_connect: Optional[str]
    """
    Maximum allowed time for establishing a connection to a backend server.
    """

    timeout_tunnel: Optional[str]
    """
    Maximum allowed tunnel inactivity time after Websocket is established (takes precedence over client and server timeout).
    """

    on_marked_down_action: Optional[OnMarkedDownAction]
    """
    Action to take when a backend server is marked as down.
    """

    proxy_protocol: Optional[ProxyProtocol]
    """
    Protocol to use between the Load Balancer and backend servers. Allows the backend servers to be informed of the client's real IP address. The PROXY protocol must be supported by the backend servers' software.
    """

    failover_host: Optional[str]
    """
    Scaleway Object Storage bucket website to be served as failover if all backend servers are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
    """

    ssl_bridging: Optional[bool]
    """
    Defines whether to enable SSL bridging between the Load Balancer and backend servers.
    """

    ignore_ssl_server_verify: Optional[bool]
    """
    Defines whether the server certificate verification should be ignored.
    """

    redispatch_attempt_count: Optional[int]
    """
    Whether to use another backend server on each attempt.
    """

    max_retries: Optional[int]
    """
    Number of retries when a backend server connection failed.
    """

    max_connections: Optional[int]
    """
    Maximum number of connections allowed per backend server.
    """

    timeout_queue: Optional[str]
    """
    Maximum time for a request to be left pending in queue when `max_connections` is reached.
    """


@dataclass
class CreateCertificateRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the certificate.
    """

    letsencrypt: Optional[CreateCertificateRequestLetsencryptConfig]

    custom_certificate: Optional[CreateCertificateRequestCustomCertificate]


@dataclass
class CreateFrontendRequest:
    inbound_port: int
    """
    Port the frontend should listen on.
    """

    lb_id: str
    """
    Load Balancer ID (ID of the Load Balancer to attach the frontend to).
    """

    backend_id: str
    """
    Backend ID (ID of the backend the frontend should pass traffic to).
    """

    enable_http3: bool
    """
    Defines whether to enable HTTP/3 protocol on the frontend.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the frontend.
    """

    timeout_client: Optional[str]
    """
    Maximum allowed inactivity time on the client side.
    """

    certificate_id: Optional[str]
    """
    Certificate ID, deprecated in favor of certificate_ids array.
    """

    certificate_ids: Optional[List[str]]
    """
    List of SSL/TLS certificate IDs to bind to the frontend.
    """

    connection_rate_limit: Optional[int]
    """
    Rate limit for new connections established on this frontend. Use 0 value to disable, else value is connections per second.
    """


@dataclass
class CreateIpRequest:
    is_ipv6: bool
    """
    If true, creates a Flexible IP with an ipv6 address.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    reverse: Optional[str]
    """
    Reverse DNS (domain name) for the IP address.
    """

    tags: Optional[List[str]]
    """
    List of tags for the IP.
    """

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class CreateLbRequest:
    description: str
    """
    Description for the Load Balancer.
    """

    type_: str
    """
    Load Balancer commercial offer type. Use the Load Balancer types endpoint to retrieve a list of available offer types.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name for the Load Balancer.
    """

    ip_id: Optional[str]
    """
    ID of an existing flexible IP address to attach to the Load Balancer.
    """

    assign_flexible_ip: Optional[bool]
    """
    Defines whether to automatically assign a flexible public IP to the Load Balancer. Default value is `true` (assign).
    """

    assign_flexible_ipv6: Optional[bool]
    """
    Defines whether to automatically assign a flexible public IPv6 to the Load Balancer. Default value is `false` (do not assign).
    """

    ip_ids: Optional[List[str]]
    """
    List of IP IDs to attach to the Load Balancer.
    """

    tags: Optional[List[str]]
    """
    List of tags for the Load Balancer.
    """

    ssl_compatibility_level: Optional[SSLCompatibilityLevel]
    """
    Determines the minimal SSL version which needs to be supported on the client side, in an SSL/TLS offloading context. Intermediate is suitable for general-purpose servers with a variety of clients, recommended for almost all systems. Modern is suitable for services with clients that support TLS 1.3 and do not need backward compatibility. Old is compatible with a small number of very old clients and should be used only as a last resort.
    """

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class CreateRouteRequest:
    frontend_id: str
    """
    ID of the source frontend to create the route on.
    """

    backend_id: str
    """
    ID of the target backend for the route.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    match: Optional[RouteMatch]
    """
    Object defining the match condition for a route to be applied. If an incoming client session matches the specified condition (i.e. it has a matching SNI value or HTTP Host header value), it will be passed to the target backend.
    """


@dataclass
class CreateSubscriberRequest:
    """
    Create a new alert subscriber (webhook or email).
    """

    name: str
    """
    Subscriber name.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]

    organization_id: Optional[str]

    email_config: Optional[SubscriberEmailConfig]

    webhook_config: Optional[SubscriberWebhookConfig]


@dataclass
class DeleteAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteBackendRequest:
    backend_id: str
    """
    ID of the backend to delete.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteFrontendRequest:
    frontend_id: str
    """
    ID of the frontend to delete.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteLbRequest:
    lb_id: str
    """
    ID of the Load Balancer to delete.
    """

    release_ip: bool
    """
    Defines whether the Load Balancer's flexible IP should be deleted. Set to true to release the flexible IP, or false to keep it available in your account for future Load Balancers.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteRouteRequest:
    route_id: str
    """
    Route ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DetachPrivateNetworkRequest:
    lb_id: str
    """
    Load balancer ID.
    """

    private_network_id: str
    """
    Set your instance private network id.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetBackendRequest:
    backend_id: str
    """
    Backend ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetFrontendRequest:
    frontend_id: str
    """
    Frontend ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetLbStatsRequest:
    """
    Get Load Balancer stats.
    """

    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    backend_id: Optional[str]
    """
    ID of the backend.
    """


@dataclass
class GetRouteRequest:
    route_id: str
    """
    Route ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class LbStats:
    backend_servers_stats: List[BackendServerStats]
    """
    List of objects containing Load Balancer statistics.
    """


@dataclass
class ListAclResponse:
    acls: List[Acl]
    """
    List of ACL objects.
    """

    total_count: int
    """
    The total number of objects.
    """


@dataclass
class ListAclsRequest:
    frontend_id: str
    """
    Frontend ID (ACLs attached to this frontend will be returned in the response).
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListAclRequestOrderBy]
    """
    Sort order of ACLs in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of ACLs to return.
    """

    name: Optional[str]
    """
    ACL name to filter for.
    """


@dataclass
class ListBackendStatsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of items to return.
    """

    backend_id: Optional[str]
    """
    ID of the backend.
    """


@dataclass
class ListBackendStatsResponse:
    backend_servers_stats: List[BackendServerStats]
    """
    List of objects containing backend server statistics.
    """

    total_count: int
    """
    The total number of objects.
    """


@dataclass
class ListBackendsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name of the backend to filter for.
    """

    order_by: Optional[ListBackendsRequestOrderBy]
    """
    Sort order of backends in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of backends to return.
    """


@dataclass
class ListBackendsResponse:
    backends: List[Backend]
    """
    List of backend objects of a given Load Balancer.
    """

    total_count: int
    """
    Total count of backend objects, without pagination.
    """


@dataclass
class ListCertificatesRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListCertificatesRequestOrderBy]
    """
    Sort order of certificates in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of certificates to return.
    """

    name: Optional[str]
    """
    Certificate name to filter for, only certificates of this name will be returned.
    """


@dataclass
class ListCertificatesResponse:
    certificates: List[Certificate]
    """
    List of certificate objects.
    """

    total_count: int
    """
    The total number of objects.
    """


@dataclass
class ListFrontendsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name of the frontend to filter for.
    """

    order_by: Optional[ListFrontendsRequestOrderBy]
    """
    Sort order of frontends in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of frontends to return.
    """


@dataclass
class ListFrontendsResponse:
    frontends: List[Frontend]
    """
    List of frontend objects of a given Load Balancer.
    """

    total_count: int
    """
    Total count of frontend objects, without pagination.
    """


@dataclass
class ListIPsRequest:
    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of IP addresses to return.
    """

    ip_address: Optional[str]
    """
    IP address to filter for.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for, only Load Balancer IP addresses from this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for, only Load Balancer IP addresses from this Project will be returned.
    """

    ip_type: Optional[ListIpsRequestIpType]
    """
    IP type to filter for.
    """

    tags: Optional[List[str]]
    """
    Tag to filter for, only IPs with one or more matching tags will be returned.
    """


@dataclass
class ListIpsResponse:
    ips: List[Ip]
    """
    List of IP address objects.
    """

    total_count: int
    """
    Total count of IP address objects, without pagination.
    """


@dataclass
class ListLbPrivateNetworksRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListPrivateNetworksRequestOrderBy]
    """
    Sort order of Private Network objects in the response.
    """

    page_size: Optional[int]
    """
    Number of objects to return.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """


@dataclass
class ListLbPrivateNetworksResponse:
    private_network: List[PrivateNetwork]
    """
    List of Private Network objects attached to the Load Balancer.
    """

    total_count: int
    """
    Total number of objects in the response.
    """


@dataclass
class ListLbTypesRequest:
    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of items to return.
    """


@dataclass
class ListLbTypesResponse:
    lb_types: List[LbType]
    """
    List of Load Balancer commercial offer type objects.
    """

    total_count: int
    """
    Total number of Load Balancer offer type objects.
    """


@dataclass
class ListLbsRequest:
    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Load Balancer name to filter for.
    """

    order_by: Optional[ListLbsRequestOrderBy]
    """
    Sort order of Load Balancers in the response.
    """

    page_size: Optional[int]
    """
    Number of Load Balancers to return.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for, only Load Balancers from this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for, only Load Balancers from this Project will be returned.
    """

    tags: Optional[List[str]]
    """
    Filter by tag, only Load Balancers with one or more matching tags will be returned.
    """


@dataclass
class ListLbsResponse:
    lbs: List[Lb]
    """
    List of Load Balancer objects.
    """

    total_count: int
    """
    The total number of Load Balancer objects.
    """


@dataclass
class ListRoutesRequest:
    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListRoutesRequestOrderBy]
    """
    Sort order of routes in the response.
    """

    page_size: Optional[int]
    """
    The number of route objects to return.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    frontend_id: Optional[str]
    """
    Frontend ID to filter for, only Routes from this Frontend will be returned.
    """


@dataclass
class ListRoutesResponse:
    routes: List[Route]
    """
    List of route objects.
    """

    total_count: int
    """
    The total number of route objects.
    """


@dataclass
class ListSubscriberRequest:
    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListSubscriberRequestOrderBy]
    """
    Sort order of subscribers in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of items to return.
    """

    name: Optional[str]
    """
    Subscriber name to search for.
    """

    organization_id: Optional[str]
    """
    Filter subscribers by Organization ID.
    """

    project_id: Optional[str]
    """
    Filter subscribers by Project ID.
    """


@dataclass
class ListSubscriberResponse:
    subscribers: List[Subscriber]
    """
    List of subscriber objects.
    """

    total_count: int
    """
    The total number of objects.
    """


@dataclass
class MigrateLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    type_: str
    """
    Load Balancer type to migrate to (use the List all Load Balancer offer types endpoint to get a list of available offer types).
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class ReleaseIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class RemoveBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses to remove from backend servers.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class SetAclsResponse:
    acls: List[Acl]
    """
    List of ACL objects.
    """

    total_count: int
    """
    The total number of ACL objects.
    """


@dataclass
class SetBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses for backend servers. Any other existing backend servers will be removed.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class SubscribeToLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    subscriber_id: str
    """
    Subscriber ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class UnsubscribeFromLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class UpdateAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    name: str
    """
    ACL name.
    """

    action: AclAction
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` & `http_filter_value` are required.
    """

    description: Optional[str]
    """
    ACL description.
    """


@dataclass
class UpdateBackendRequest:
    backend_id: str
    """
    Backend ID.
    """

    name: str
    """
    Backend name.
    """

    forward_protocol: Protocol
    """
    Protocol to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port: int
    """
    Port to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port_algorithm: ForwardPortAlgorithm
    """
    Load balancing algorithm to be used when determining which backend server to forward new traffic to.
    """

    sticky_sessions: StickySessionsType
    """
    Defines whether to activate sticky sessions (binding a particular session to a particular backend server) and the method to use if so. None disables sticky sessions. Cookie-based uses an HTTP cookie to stick a session to a backend server. Table-based uses the source (client) IP address to stick a session to a backend server.
    """

    sticky_sessions_cookie_name: str
    """
    Cookie name for cookie-based sticky sessions.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    send_proxy_v2: Optional[bool]
    """
    Deprecated in favor of proxy_protocol field.
    """

    timeout_server: Optional[str]
    """
    Maximum allowed time for a backend server to process a request.
    """

    timeout_connect: Optional[str]
    """
    Maximum allowed time for establishing a connection to a backend server.
    """

    timeout_tunnel: Optional[str]
    """
    Maximum allowed tunnel inactivity time after Websocket is established (takes precedence over client and server timeout).
    """

    on_marked_down_action: Optional[OnMarkedDownAction]
    """
    Action to take when a backend server is marked as down.
    """

    proxy_protocol: Optional[ProxyProtocol]
    """
    Protocol to use between the Load Balancer and backend servers. Allows the backend servers to be informed of the client's real IP address. The PROXY protocol must be supported by the backend servers' software.
    """

    failover_host: Optional[str]
    """
    Scaleway Object Storage bucket website to be served as failover if all backend servers are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
    """

    ssl_bridging: Optional[bool]
    """
    Defines whether to enable SSL bridging between the Load Balancer and backend servers.
    """

    ignore_ssl_server_verify: Optional[bool]
    """
    Defines whether the server certificate verification should be ignored.
    """

    redispatch_attempt_count: Optional[int]
    """
    Whether to use another backend server on each attempt.
    """

    max_retries: Optional[int]
    """
    Number of retries when a backend server connection failed.
    """

    max_connections: Optional[int]
    """
    Maximum number of connections allowed per backend server.
    """

    timeout_queue: Optional[str]
    """
    Maximum time for a request to be left pending in queue when `max_connections` is reached.
    """


@dataclass
class UpdateCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    name: str
    """
    Certificate name.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class UpdateFrontendRequest:
    frontend_id: str
    """
    Frontend ID.
    """

    name: str
    """
    Frontend name.
    """

    inbound_port: int
    """
    Port the frontend should listen on.
    """

    backend_id: str
    """
    Backend ID (ID of the backend the frontend should pass traffic to).
    """

    enable_http3: bool
    """
    Defines whether to enable HTTP/3 protocol on the frontend.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    timeout_client: Optional[str]
    """
    Maximum allowed inactivity time on the client side.
    """

    certificate_id: Optional[str]
    """
    Certificate ID, deprecated in favor of certificate_ids array.
    """

    certificate_ids: Optional[List[str]]
    """
    List of SSL/TLS certificate IDs to bind to the frontend.
    """

    connection_rate_limit: Optional[int]
    """
    Rate limit for new connections established on this frontend. Use 0 value to disable, else value is connections per second.
    """


@dataclass
class UpdateHealthCheckRequest:
    port: int
    """
    Port to use for the backend server health check.
    """

    check_max_retries: int
    """
    Number of consecutive unsuccessful health checks after which the server will be considered dead.
    """

    backend_id: str
    """
    Backend ID.
    """

    check_send_proxy: bool
    """
    Defines whether proxy protocol should be activated for the health check.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks.
    """

    check_timeout: Optional[str]
    """
    Maximum time a backend server has to reply to the health check.
    """

    transient_check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks when a backend server is in a transient state (going UP or DOWN).
    """

    tcp_config: Optional[HealthCheckTcpConfig]

    mysql_config: Optional[HealthCheckMysqlConfig]

    pgsql_config: Optional[HealthCheckPgsqlConfig]

    ldap_config: Optional[HealthCheckLdapConfig]

    redis_config: Optional[HealthCheckRedisConfig]

    http_config: Optional[HealthCheckHttpConfig]

    https_config: Optional[HealthCheckHttpsConfig]


@dataclass
class UpdateIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    reverse: Optional[str]
    """
    Reverse DNS (domain name) for the IP address.
    """

    lb_id: Optional[str]
    """
    ID of the server on which to attach the flexible IP.
    """

    tags: Optional[List[str]]
    """
    List of tags for the IP.
    """


@dataclass
class UpdateLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    name: str
    """
    Load Balancer name.
    """

    description: str
    """
    Load Balancer description.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    tags: Optional[List[str]]
    """
    List of tags for the Load Balancer.
    """

    ssl_compatibility_level: Optional[SSLCompatibilityLevel]
    """
    Determines the minimal SSL version which needs to be supported on the client side, in an SSL/TLS offloading context. Intermediate is suitable for general-purpose servers with a variety of clients, recommended for almost all systems. Modern is suitable for services with clients that support TLS 1.3 and don't need backward compatibility. Old is compatible with a small number of very old clients and should be used only as a last resort.
    """


@dataclass
class UpdateRouteRequest:
    route_id: str
    """
    Route ID.
    """

    backend_id: str
    """
    ID of the target backend for the route.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    match: Optional[RouteMatch]
    """
    Object defining the match condition for a route to be applied. If an incoming client session matches the specified condition (i.e. it has a matching SNI value or HTTP Host header value), it will be passed to the target backend.
    """


@dataclass
class UpdateSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    name: str
    """
    Subscriber name.
    """

    region: Optional[ScwRegion]
    """
    Region to target. If none is passed will use default region from the config.
    """

    email_config: Optional[SubscriberEmailConfig]

    webhook_config: Optional[SubscriberWebhookConfig]


@dataclass
class ZonedApiAddBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses to add to backend servers.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiAttachPrivateNetworkRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ipam_ids: Optional[List[str]]
    """
    IPAM ID of a pre-reserved IP address to assign to the Load Balancer on this Private Network. In the future, it will be possible to specify multiple IPs in this field (IPv4 and IPv6), for now only one ID of an IPv4 address is expected. When null, a new private IP address is created for the Load Balancer on this Private Network.
    """

    static_config: Optional[PrivateNetworkStaticConfig]

    dhcp_config: Optional[PrivateNetworkDHCPConfig]

    ipam_config: Optional[PrivateNetworkIpamConfig]


@dataclass
class ZonedApiCreateAclRequest:
    """
    Add an ACL to a Load Balancer frontend.
    """

    frontend_id: str
    """
    Frontend ID to attach the ACL to.
    """

    action: AclAction
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    description: str
    """
    ACL description.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    ACL name.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` & `http_filter_value` are required.
    """


@dataclass
class ZonedApiCreateBackendRequest:
    forward_protocol: Protocol
    """
    Protocol to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port: int
    """
    Port to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port_algorithm: ForwardPortAlgorithm
    """
    Load balancing algorithm to be used when determining which backend server to forward new traffic to.
    """

    sticky_sessions: StickySessionsType
    """
    Defines whether to activate sticky sessions (binding a particular session to a particular backend server) and the method to use if so. None disables sticky sessions. Cookie-based uses an HTTP cookie TO stick a session to a backend server. Table-based uses the source (client) IP address to stick a session to a backend server.
    """

    sticky_sessions_cookie_name: str
    """
    Cookie name for cookie-based sticky sessions.
    """

    lb_id: str
    """
    Load Balancer ID.
    """

    health_check: HealthCheck
    """
    Object defining the health check to be carried out by the backend when checking the status and health of backend servers.
    """

    server_ip: List[str]
    """
    List of backend server IP addresses (IPv4 or IPv6) the backend should forward traffic to.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the backend.
    """

    send_proxy_v2: Optional[bool]
    """
    Deprecated in favor of proxy_protocol field.
    """

    timeout_server: Optional[str]
    """
    Maximum allowed time for a backend server to process a request.
    """

    timeout_connect: Optional[str]
    """
    Maximum allowed time for establishing a connection to a backend server.
    """

    timeout_tunnel: Optional[str]
    """
    Maximum allowed tunnel inactivity time after Websocket is established (takes precedence over client and server timeout).
    """

    on_marked_down_action: Optional[OnMarkedDownAction]
    """
    Action to take when a backend server is marked as down.
    """

    proxy_protocol: Optional[ProxyProtocol]
    """
    Protocol to use between the Load Balancer and backend servers. Allows the backend servers to be informed of the client's real IP address. The PROXY protocol must be supported by the backend servers' software.
    """

    failover_host: Optional[str]
    """
    Scaleway Object Storage bucket website to be served as failover if all backend servers are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
    """

    ssl_bridging: Optional[bool]
    """
    Defines whether to enable SSL bridging between the Load Balancer and backend servers.
    """

    ignore_ssl_server_verify: Optional[bool]
    """
    Defines whether the server certificate verification should be ignored.
    """

    redispatch_attempt_count: Optional[int]
    """
    Whether to use another backend server on each attempt.
    """

    max_retries: Optional[int]
    """
    Number of retries when a backend server connection failed.
    """

    max_connections: Optional[int]
    """
    Maximum number of connections allowed per backend server.
    """

    timeout_queue: Optional[str]
    """
    Maximum time for a request to be left pending in queue when `max_connections` is reached.
    """


@dataclass
class ZonedApiCreateCertificateRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the certificate.
    """

    letsencrypt: Optional[CreateCertificateRequestLetsencryptConfig]

    custom_certificate: Optional[CreateCertificateRequestCustomCertificate]


@dataclass
class ZonedApiCreateFrontendRequest:
    inbound_port: int
    """
    Port the frontend should listen on.
    """

    lb_id: str
    """
    Load Balancer ID (ID of the Load Balancer to attach the frontend to).
    """

    backend_id: str
    """
    Backend ID (ID of the backend the frontend should pass traffic to).
    """

    enable_http3: bool
    """
    Defines whether to enable HTTP/3 protocol on the frontend.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the frontend.
    """

    timeout_client: Optional[str]
    """
    Maximum allowed inactivity time on the client side.
    """

    certificate_id: Optional[str]
    """
    Certificate ID, deprecated in favor of certificate_ids array.
    """

    certificate_ids: Optional[List[str]]
    """
    List of SSL/TLS certificate IDs to bind to the frontend.
    """

    connection_rate_limit: Optional[int]
    """
    Rate limit for new connections established on this frontend. Use 0 value to disable, else value is connections per second.
    """


@dataclass
class ZonedApiCreateIpRequest:
    is_ipv6: bool
    """
    If true, creates a Flexible IP with an ipv6 address.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    reverse: Optional[str]
    """
    Reverse DNS (domain name) for the IP address.
    """

    tags: Optional[List[str]]
    """
    List of tags for the IP.
    """

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class ZonedApiCreateLbRequest:
    description: str
    """
    Description for the Load Balancer.
    """

    type_: str
    """
    Load Balancer commercial offer type. Use the Load Balancer types endpoint to retrieve a list of available offer types.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name for the Load Balancer.
    """

    ip_id: Optional[str]
    """
    ID of an existing flexible IP address to attach to the Load Balancer.
    """

    assign_flexible_ip: Optional[bool]
    """
    Defines whether to automatically assign a flexible public IP to the Load Balancer. Default value is `true` (assign).
    """

    assign_flexible_ipv6: Optional[bool]
    """
    Defines whether to automatically assign a flexible public IPv6 to the Load Balancer. Default value is `false` (do not assign).
    """

    ip_ids: Optional[List[str]]
    """
    List of IP IDs to attach to the Load Balancer.
    """

    tags: Optional[List[str]]
    """
    List of tags for the Load Balancer.
    """

    ssl_compatibility_level: Optional[SSLCompatibilityLevel]
    """
    Determines the minimal SSL version which needs to be supported on the client side, in an SSL/TLS offloading context. Intermediate is suitable for general-purpose servers with a variety of clients, recommended for almost all systems. Modern is suitable for services with clients that support TLS 1.3 and do not need backward compatibility. Old is compatible with a small number of very old clients and should be used only as a last resort.
    """

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class ZonedApiCreateRouteRequest:
    frontend_id: str
    """
    ID of the source frontend to create the route on.
    """

    backend_id: str
    """
    ID of the target backend for the route.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    match: Optional[RouteMatch]
    """
    Object defining the match condition for a route to be applied. If an incoming client session matches the specified condition (i.e. it has a matching SNI value or HTTP Host header value), it will be passed to the target backend.
    """


@dataclass
class ZonedApiCreateSubscriberRequest:
    """
    Create a new alert subscriber (webhook or email).
    """

    name: str
    """
    Subscriber name.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]

    organization_id: Optional[str]

    email_config: Optional[SubscriberEmailConfig]

    webhook_config: Optional[SubscriberWebhookConfig]


@dataclass
class ZonedApiDeleteAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteBackendRequest:
    backend_id: str
    """
    ID of the backend to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteFrontendRequest:
    frontend_id: str
    """
    ID of the frontend to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteLbRequest:
    lb_id: str
    """
    ID of the Load Balancer to delete.
    """

    release_ip: bool
    """
    Defines whether the Load Balancer's flexible IP should be deleted. Set to true to release the flexible IP, or false to keep it available in your account for future Load Balancers.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteRouteRequest:
    route_id: str
    """
    Route ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDeleteSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiDetachPrivateNetworkRequest:
    lb_id: str
    """
    Load balancer ID.
    """

    private_network_id: str
    """
    Set your instance private network id.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetBackendRequest:
    backend_id: str
    """
    Backend ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetFrontendRequest:
    frontend_id: str
    """
    Frontend ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetLbStatsRequest:
    """
    Get Load Balancer stats.
    """

    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    backend_id: Optional[str]
    """
    ID of the backend.
    """


@dataclass
class ZonedApiGetRouteRequest:
    route_id: str
    """
    Route ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiGetSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiListAclsRequest:
    frontend_id: str
    """
    Frontend ID (ACLs attached to this frontend will be returned in the response).
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListAclRequestOrderBy]
    """
    Sort order of ACLs in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of ACLs to return.
    """

    name: Optional[str]
    """
    ACL name to filter for.
    """


@dataclass
class ZonedApiListBackendStatsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of items to return.
    """

    backend_id: Optional[str]
    """
    ID of the backend.
    """


@dataclass
class ZonedApiListBackendsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name of the backend to filter for.
    """

    order_by: Optional[ListBackendsRequestOrderBy]
    """
    Sort order of backends in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of backends to return.
    """


@dataclass
class ZonedApiListCertificatesRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListCertificatesRequestOrderBy]
    """
    Sort order of certificates in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of certificates to return.
    """

    name: Optional[str]
    """
    Certificate name to filter for, only certificates of this name will be returned.
    """


@dataclass
class ZonedApiListFrontendsRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name of the frontend to filter for.
    """

    order_by: Optional[ListFrontendsRequestOrderBy]
    """
    Sort order of frontends in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of frontends to return.
    """


@dataclass
class ZonedApiListIPsRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Number of IP addresses to return.
    """

    ip_address: Optional[str]
    """
    IP address to filter for.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for, only Load Balancer IP addresses from this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for, only Load Balancer IP addresses from this Project will be returned.
    """

    ip_type: Optional[ListIpsRequestIpType]
    """
    IP type to filter for.
    """

    tags: Optional[List[str]]
    """
    Tag to filter for, only IPs with one or more matching tags will be returned.
    """


@dataclass
class ZonedApiListLbPrivateNetworksRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListPrivateNetworksRequestOrderBy]
    """
    Sort order of Private Network objects in the response.
    """

    page_size: Optional[int]
    """
    Number of objects to return.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """


@dataclass
class ZonedApiListLbTypesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of items to return.
    """


@dataclass
class ZonedApiListLbsRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Load Balancer name to filter for.
    """

    order_by: Optional[ListLbsRequestOrderBy]
    """
    Sort order of Load Balancers in the response.
    """

    page_size: Optional[int]
    """
    Number of Load Balancers to return.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    organization_id: Optional[str]
    """
    Organization ID to filter for, only Load Balancers from this Organization will be returned.
    """

    project_id: Optional[str]
    """
    Project ID to filter for, only Load Balancers from this Project will be returned.
    """

    tags: Optional[List[str]]
    """
    Filter by tag, only Load Balancers with one or more matching tags will be returned.
    """


@dataclass
class ZonedApiListRoutesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListRoutesRequestOrderBy]
    """
    Sort order of routes in the response.
    """

    page_size: Optional[int]
    """
    The number of route objects to return.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    frontend_id: Optional[str]
    """
    Frontend ID to filter for, only Routes from this Frontend will be returned.
    """


@dataclass
class ZonedApiListSubscriberRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListSubscriberRequestOrderBy]
    """
    Sort order of subscribers in the response.
    """

    page: Optional[int]
    """
    The page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    The number of items to return.
    """

    name: Optional[str]
    """
    Subscriber name to search for.
    """

    organization_id: Optional[str]
    """
    Filter subscribers by Organization ID.
    """

    project_id: Optional[str]
    """
    Filter subscribers by Project ID.
    """


@dataclass
class ZonedApiMigrateLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    type_: str
    """
    Load Balancer type to migrate to (use the List all Load Balancer offer types endpoint to get a list of available offer types).
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiReleaseIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiRemoveBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses to remove from backend servers.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiSetAclsRequest:
    acls: List[AclSpec]
    """
    List of ACLs for this frontend. Any other existing ACLs on this frontend will be removed.
    """

    frontend_id: str
    """
    Frontend ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiSetBackendServersRequest:
    backend_id: str
    """
    Backend ID.
    """

    server_ip: List[str]
    """
    List of IP addresses for backend servers. Any other existing backend servers will be removed.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiSubscribeToLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    subscriber_id: str
    """
    Subscriber ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiUnsubscribeFromLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiUpdateAclRequest:
    acl_id: str
    """
    ACL ID.
    """

    name: str
    """
    ACL name.
    """

    action: AclAction
    """
    Action to take when incoming traffic matches an ACL filter.
    """

    index: int
    """
    Priority of this ACL (ACLs are applied in ascending order, 0 is the first ACL executed).
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    match: Optional[AclMatch]
    """
    ACL match filter object. One of `ip_subnet` or `http_filter` & `http_filter_value` are required.
    """

    description: Optional[str]
    """
    ACL description.
    """


@dataclass
class ZonedApiUpdateBackendRequest:
    backend_id: str
    """
    Backend ID.
    """

    name: str
    """
    Backend name.
    """

    forward_protocol: Protocol
    """
    Protocol to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port: int
    """
    Port to be used by the backend when forwarding traffic to backend servers.
    """

    forward_port_algorithm: ForwardPortAlgorithm
    """
    Load balancing algorithm to be used when determining which backend server to forward new traffic to.
    """

    sticky_sessions: StickySessionsType
    """
    Defines whether to activate sticky sessions (binding a particular session to a particular backend server) and the method to use if so. None disables sticky sessions. Cookie-based uses an HTTP cookie to stick a session to a backend server. Table-based uses the source (client) IP address to stick a session to a backend server.
    """

    sticky_sessions_cookie_name: str
    """
    Cookie name for cookie-based sticky sessions.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    send_proxy_v2: Optional[bool]
    """
    Deprecated in favor of proxy_protocol field.
    """

    timeout_server: Optional[str]
    """
    Maximum allowed time for a backend server to process a request.
    """

    timeout_connect: Optional[str]
    """
    Maximum allowed time for establishing a connection to a backend server.
    """

    timeout_tunnel: Optional[str]
    """
    Maximum allowed tunnel inactivity time after Websocket is established (takes precedence over client and server timeout).
    """

    on_marked_down_action: Optional[OnMarkedDownAction]
    """
    Action to take when a backend server is marked as down.
    """

    proxy_protocol: Optional[ProxyProtocol]
    """
    Protocol to use between the Load Balancer and backend servers. Allows the backend servers to be informed of the client's real IP address. The PROXY protocol must be supported by the backend servers' software.
    """

    failover_host: Optional[str]
    """
    Scaleway Object Storage bucket website to be served as failover if all backend servers are down, e.g. failover-website.s3-website.fr-par.scw.cloud.
    """

    ssl_bridging: Optional[bool]
    """
    Defines whether to enable SSL bridging between the Load Balancer and backend servers.
    """

    ignore_ssl_server_verify: Optional[bool]
    """
    Defines whether the server certificate verification should be ignored.
    """

    redispatch_attempt_count: Optional[int]
    """
    Whether to use another backend server on each attempt.
    """

    max_retries: Optional[int]
    """
    Number of retries when a backend server connection failed.
    """

    max_connections: Optional[int]
    """
    Maximum number of connections allowed per backend server.
    """

    timeout_queue: Optional[str]
    """
    Maximum time for a request to be left pending in queue when `max_connections` is reached.
    """


@dataclass
class ZonedApiUpdateCertificateRequest:
    certificate_id: str
    """
    Certificate ID.
    """

    name: str
    """
    Certificate name.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ZonedApiUpdateFrontendRequest:
    frontend_id: str
    """
    Frontend ID.
    """

    name: str
    """
    Frontend name.
    """

    inbound_port: int
    """
    Port the frontend should listen on.
    """

    backend_id: str
    """
    Backend ID (ID of the backend the frontend should pass traffic to).
    """

    enable_http3: bool
    """
    Defines whether to enable HTTP/3 protocol on the frontend.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    timeout_client: Optional[str]
    """
    Maximum allowed inactivity time on the client side.
    """

    certificate_id: Optional[str]
    """
    Certificate ID, deprecated in favor of certificate_ids array.
    """

    certificate_ids: Optional[List[str]]
    """
    List of SSL/TLS certificate IDs to bind to the frontend.
    """

    connection_rate_limit: Optional[int]
    """
    Rate limit for new connections established on this frontend. Use 0 value to disable, else value is connections per second.
    """


@dataclass
class ZonedApiUpdateHealthCheckRequest:
    port: int
    """
    Port to use for the backend server health check.
    """

    check_max_retries: int
    """
    Number of consecutive unsuccessful health checks after which the server will be considered dead.
    """

    backend_id: str
    """
    Backend ID.
    """

    check_send_proxy: bool
    """
    Defines whether proxy protocol should be activated for the health check.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks.
    """

    check_timeout: Optional[str]
    """
    Maximum time a backend server has to reply to the health check.
    """

    transient_check_delay: Optional[str]
    """
    Time to wait between two consecutive health checks when a backend server is in a transient state (going UP or DOWN).
    """

    tcp_config: Optional[HealthCheckTcpConfig]

    mysql_config: Optional[HealthCheckMysqlConfig]

    pgsql_config: Optional[HealthCheckPgsqlConfig]

    ldap_config: Optional[HealthCheckLdapConfig]

    redis_config: Optional[HealthCheckRedisConfig]

    http_config: Optional[HealthCheckHttpConfig]

    https_config: Optional[HealthCheckHttpsConfig]


@dataclass
class ZonedApiUpdateIpRequest:
    ip_id: str
    """
    IP address ID.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    reverse: Optional[str]
    """
    Reverse DNS (domain name) for the IP address.
    """

    lb_id: Optional[str]
    """
    ID of the server on which to attach the flexible IP.
    """

    tags: Optional[List[str]]
    """
    List of tags for the IP.
    """


@dataclass
class ZonedApiUpdateLbRequest:
    lb_id: str
    """
    Load Balancer ID.
    """

    name: str
    """
    Load Balancer name.
    """

    description: str
    """
    Load Balancer description.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    tags: Optional[List[str]]
    """
    List of tags for the Load Balancer.
    """

    ssl_compatibility_level: Optional[SSLCompatibilityLevel]
    """
    Determines the minimal SSL version which needs to be supported on the client side, in an SSL/TLS offloading context. Intermediate is suitable for general-purpose servers with a variety of clients, recommended for almost all systems. Modern is suitable for services with clients that support TLS 1.3 and don't need backward compatibility. Old is compatible with a small number of very old clients and should be used only as a last resort.
    """


@dataclass
class ZonedApiUpdateRouteRequest:
    route_id: str
    """
    Route ID.
    """

    backend_id: str
    """
    ID of the target backend for the route.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    match: Optional[RouteMatch]
    """
    Object defining the match condition for a route to be applied. If an incoming client session matches the specified condition (i.e. it has a matching SNI value or HTTP Host header value), it will be passed to the target backend.
    """


@dataclass
class ZonedApiUpdateSubscriberRequest:
    subscriber_id: str
    """
    Subscriber ID.
    """

    name: str
    """
    Subscriber name.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    email_config: Optional[SubscriberEmailConfig]

    webhook_config: Optional[SubscriberWebhookConfig]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ContactEmailStatus
from .types import ContactExtensionFRMode
from .types import ContactExtensionNLLegalForm
from .types import ContactLegalForm
from .types import ContactStatus
from .types import DNSZoneStatus
from .content import DNS_ZONE_TRANSIENT_STATUSES
from .types import DSRecordAlgorithm
from .types import DSRecordDigestType
from .types import DomainFeatureStatus
from .content import DOMAIN_FEATURE_TRANSIENT_STATUSES
from .types import DomainRegistrationStatusTransferStatus
from .content import DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES
from .types import DomainStatus
from .content import DOMAIN_TRANSIENT_STATUSES
from .types import HostStatus
from .content import HOST_TRANSIENT_STATUSES
from .types import LinkedProduct
from .types import ListContactsRequestRole
from .types import ListDNSZoneRecordsRequestOrderBy
from .types import ListDNSZonesRequestOrderBy
from .types import ListDomainsRequestOrderBy
from .types import ListRenewableDomainsRequestOrderBy
from .types import ListTasksRequestOrderBy
from .types import ListTldsRequestOrderBy
from .types import RawFormat
from .types import RecordHTTPServiceConfigStrategy
from .types import RecordType
from .types import RenewableDomainStatus
from .types import SSLCertificateStatus
from .content import SSL_CERTIFICATE_TRANSIENT_STATUSES
from .types import TaskStatus
from .content import TASK_TRANSIENT_STATUSES
from .types import TaskType
from .types import RecordGeoIPConfigMatch
from .types import RecordViewConfigView
from .types import RecordWeightedConfigWeightedIP
from .types import DSRecordPublicKey
from .types import RecordGeoIPConfig
from .types import RecordHTTPServiceConfig
from .types import RecordViewConfig
from .types import RecordWeightedConfig
from .types import ContactExtensionFRAssociationInfo
from .types import ContactExtensionFRCodeAuthAfnicInfo
from .types import ContactExtensionFRDunsInfo
from .types import ContactExtensionFRIndividualInfo
from .types import ContactExtensionFRTrademarkInfo
from .types import DSRecordDigest
from .types import Record
from .types import RecordIdentifier
from .types import ContactExtensionEU
from .types import ContactExtensionFR
from .types import ContactExtensionNL
from .types import ContactQuestion
from .types import TldOffer
from .types import DSRecord
from .types import RecordChangeAdd
from .types import RecordChangeClear
from .types import RecordChangeDelete
from .types import RecordChangeSet
from .types import ImportRawDNSZoneRequestTsigKey
from .types import Contact
from .types import ContactRolesRoles
from .types import DomainRegistrationStatusExternalDomain
from .types import DomainRegistrationStatusTransfer
from .types import Tld
from .types import NewContact
from .types import CheckContactsCompatibilityResponseContactCheckResult
from .types import DNSZone
from .types import DomainDNSSEC
from .types import RecordChange
from .types import ImportProviderDNSZoneRequestOnlineV1
from .types import ImportRawDNSZoneRequestAXFRSource
from .types import ImportRawDNSZoneRequestBindSource
from .types import ContactRoles
from .types import Nameserver
from .types import DNSZoneVersion
from .types import Host
from .types import DomainSummary
from .types import RenewableDomain
from .types import SSLCertificate
from .types import Task
from .types import TransferInDomainRequestTransferRequest
from .types import UpdateContactRequestQuestion
from .types import AvailableDomain
from .types import CheckContactsCompatibilityResponse
from .types import ClearDNSZoneRecordsRequest
from .types import ClearDNSZoneRecordsResponse
from .types import CloneDNSZoneRequest
from .types import CreateDNSZoneRequest
from .types import CreateSSLCertificateRequest
from .types import DeleteDNSZoneRequest
from .types import DeleteDNSZoneResponse
from .types import DeleteDNSZoneTsigKeyRequest
from .types import DeleteExternalDomainResponse
from .types import DeleteSSLCertificateRequest
from .types import DeleteSSLCertificateResponse
from .types import Domain
from .types import ExportRawDNSZoneRequest
from .types import GetDNSZoneTsigKeyRequest
from .types import GetDNSZoneTsigKeyResponse
from .types import GetDNSZoneVersionDiffRequest
from .types import GetDNSZoneVersionDiffResponse
from .types import GetDomainAuthCodeResponse
from .types import GetSSLCertificateRequest
from .types import ImportProviderDNSZoneRequest
from .types import ImportProviderDNSZoneResponse
from .types import ImportRawDNSZoneRequest
from .types import ImportRawDNSZoneResponse
from .types import ListContactsResponse
from .types import ListDNSZoneNameserversRequest
from .types import ListDNSZoneNameserversResponse
from .types import ListDNSZoneRecordsRequest
from .types import ListDNSZoneRecordsResponse
from .types import ListDNSZoneVersionRecordsRequest
from .types import ListDNSZoneVersionRecordsResponse
from .types import ListDNSZoneVersionsRequest
from .types import ListDNSZoneVersionsResponse
from .types import ListDNSZonesRequest
from .types import ListDNSZonesResponse
from .types import ListDomainHostsResponse
from .types import ListDomainsResponse
from .types import ListRenewableDomainsResponse
from .types import ListSSLCertificatesRequest
from .types import ListSSLCertificatesResponse
from .types import ListTasksResponse
from .types import ListTldsResponse
from .types import OrderResponse
from .types import RefreshDNSZoneRequest
from .types import RefreshDNSZoneResponse
from .types import RegisterExternalDomainResponse
from .types import RegistrarApiBuyDomainsRequest
from .types import RegistrarApiCheckContactsCompatibilityRequest
from .types import RegistrarApiCreateDomainHostRequest
from .types import RegistrarApiDeleteDomainHostRequest
from .types import RegistrarApiDeleteExternalDomainRequest
from .types import RegistrarApiDisableDomainAutoRenewRequest
from .types import RegistrarApiDisableDomainDNSSECRequest
from .types import RegistrarApiEnableDomainAutoRenewRequest
from .types import RegistrarApiEnableDomainDNSSECRequest
from .types import RegistrarApiGetContactRequest
from .types import RegistrarApiGetDomainAuthCodeRequest
from .types import RegistrarApiGetDomainRequest
from .types import RegistrarApiListContactsRequest
from .types import RegistrarApiListDomainHostsRequest
from .types import RegistrarApiListDomainsRequest
from .types import RegistrarApiListRenewableDomainsRequest
from .types import RegistrarApiListTasksRequest
from .types import RegistrarApiListTldsRequest
from .types import RegistrarApiLockDomainTransferRequest
from .types import RegistrarApiRegisterExternalDomainRequest
from .types import RegistrarApiRenewDomainsRequest
from .types import RegistrarApiSearchAvailableDomainsRequest
from .types import RegistrarApiTradeDomainRequest
from .types import RegistrarApiTransferInDomainRequest
from .types import RegistrarApiUnlockDomainTransferRequest
from .types import RegistrarApiUpdateContactRequest
from .types import RegistrarApiUpdateDomainHostRequest
from .types import RegistrarApiUpdateDomainRequest
from .types import RestoreDNSZoneVersionRequest
from .types import RestoreDNSZoneVersionResponse
from .types import SearchAvailableDomainsResponse
from .types import UpdateDNSZoneNameserversRequest
from .types import UpdateDNSZoneNameserversResponse
from .types import UpdateDNSZoneRecordsRequest
from .types import UpdateDNSZoneRecordsResponse
from .types import UpdateDNSZoneRequest
from .api import DomainV2Beta1API
from .api import DomainV2Beta1RegistrarAPI

__all__ = [
    "ContactEmailStatus",
    "ContactExtensionFRMode",
    "ContactExtensionNLLegalForm",
    "ContactLegalForm",
    "ContactStatus",
    "DNSZoneStatus",
    "DNS_ZONE_TRANSIENT_STATUSES",
    "DSRecordAlgorithm",
    "DSRecordDigestType",
    "DomainFeatureStatus",
    "DOMAIN_FEATURE_TRANSIENT_STATUSES",
    "DomainRegistrationStatusTransferStatus",
    "DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES",
    "DomainStatus",
    "DOMAIN_TRANSIENT_STATUSES",
    "HostStatus",
    "HOST_TRANSIENT_STATUSES",
    "LinkedProduct",
    "ListContactsRequestRole",
    "ListDNSZoneRecordsRequestOrderBy",
    "ListDNSZonesRequestOrderBy",
    "ListDomainsRequestOrderBy",
    "ListRenewableDomainsRequestOrderBy",
    "ListTasksRequestOrderBy",
    "ListTldsRequestOrderBy",
    "RawFormat",
    "RecordHTTPServiceConfigStrategy",
    "RecordType",
    "RenewableDomainStatus",
    "SSLCertificateStatus",
    "SSL_CERTIFICATE_TRANSIENT_STATUSES",
    "TaskStatus",
    "TASK_TRANSIENT_STATUSES",
    "TaskType",
    "RecordGeoIPConfigMatch",
    "RecordViewConfigView",
    "RecordWeightedConfigWeightedIP",
    "DSRecordPublicKey",
    "RecordGeoIPConfig",
    "RecordHTTPServiceConfig",
    "RecordViewConfig",
    "RecordWeightedConfig",
    "ContactExtensionFRAssociationInfo",
    "ContactExtensionFRCodeAuthAfnicInfo",
    "ContactExtensionFRDunsInfo",
    "ContactExtensionFRIndividualInfo",
    "ContactExtensionFRTrademarkInfo",
    "DSRecordDigest",
    "Record",
    "RecordIdentifier",
    "ContactExtensionEU",
    "ContactExtensionFR",
    "ContactExtensionNL",
    "ContactQuestion",
    "TldOffer",
    "DSRecord",
    "RecordChangeAdd",
    "RecordChangeClear",
    "RecordChangeDelete",
    "RecordChangeSet",
    "ImportRawDNSZoneRequestTsigKey",
    "Contact",
    "ContactRolesRoles",
    "DomainRegistrationStatusExternalDomain",
    "DomainRegistrationStatusTransfer",
    "Tld",
    "NewContact",
    "CheckContactsCompatibilityResponseContactCheckResult",
    "DNSZone",
    "DomainDNSSEC",
    "RecordChange",
    "ImportProviderDNSZoneRequestOnlineV1",
    "ImportRawDNSZoneRequestAXFRSource",
    "ImportRawDNSZoneRequestBindSource",
    "ContactRoles",
    "Nameserver",
    "DNSZoneVersion",
    "Host",
    "DomainSummary",
    "RenewableDomain",
    "SSLCertificate",
    "Task",
    "TransferInDomainRequestTransferRequest",
    "UpdateContactRequestQuestion",
    "AvailableDomain",
    "CheckContactsCompatibilityResponse",
    "ClearDNSZoneRecordsRequest",
    "ClearDNSZoneRecordsResponse",
    "CloneDNSZoneRequest",
    "CreateDNSZoneRequest",
    "CreateSSLCertificateRequest",
    "DeleteDNSZoneRequest",
    "DeleteDNSZoneResponse",
    "DeleteDNSZoneTsigKeyRequest",
    "DeleteExternalDomainResponse",
    "DeleteSSLCertificateRequest",
    "DeleteSSLCertificateResponse",
    "Domain",
    "ExportRawDNSZoneRequest",
    "GetDNSZoneTsigKeyRequest",
    "GetDNSZoneTsigKeyResponse",
    "GetDNSZoneVersionDiffRequest",
    "GetDNSZoneVersionDiffResponse",
    "GetDomainAuthCodeResponse",
    "GetSSLCertificateRequest",
    "ImportProviderDNSZoneRequest",
    "ImportProviderDNSZoneResponse",
    "ImportRawDNSZoneRequest",
    "ImportRawDNSZoneResponse",
    "ListContactsResponse",
    "ListDNSZoneNameserversRequest",
    "ListDNSZoneNameserversResponse",
    "ListDNSZoneRecordsRequest",
    "ListDNSZoneRecordsResponse",
    "ListDNSZoneVersionRecordsRequest",
    "ListDNSZoneVersionRecordsResponse",
    "ListDNSZoneVersionsRequest",
    "ListDNSZoneVersionsResponse",
    "ListDNSZonesRequest",
    "ListDNSZonesResponse",
    "ListDomainHostsResponse",
    "ListDomainsResponse",
    "ListRenewableDomainsResponse",
    "ListSSLCertificatesRequest",
    "ListSSLCertificatesResponse",
    "ListTasksResponse",
    "ListTldsResponse",
    "OrderResponse",
    "RefreshDNSZoneRequest",
    "RefreshDNSZoneResponse",
    "RegisterExternalDomainResponse",
    "RegistrarApiBuyDomainsRequest",
    "RegistrarApiCheckContactsCompatibilityRequest",
    "RegistrarApiCreateDomainHostRequest",
    "RegistrarApiDeleteDomainHostRequest",
    "RegistrarApiDeleteExternalDomainRequest",
    "RegistrarApiDisableDomainAutoRenewRequest",
    "RegistrarApiDisableDomainDNSSECRequest",
    "RegistrarApiEnableDomainAutoRenewRequest",
    "RegistrarApiEnableDomainDNSSECRequest",
    "RegistrarApiGetContactRequest",
    "RegistrarApiGetDomainAuthCodeRequest",
    "RegistrarApiGetDomainRequest",
    "RegistrarApiListContactsRequest",
    "RegistrarApiListDomainHostsRequest",
    "RegistrarApiListDomainsRequest",
    "RegistrarApiListRenewableDomainsRequest",
    "RegistrarApiListTasksRequest",
    "RegistrarApiListTldsRequest",
    "RegistrarApiLockDomainTransferRequest",
    "RegistrarApiRegisterExternalDomainRequest",
    "RegistrarApiRenewDomainsRequest",
    "RegistrarApiSearchAvailableDomainsRequest",
    "RegistrarApiTradeDomainRequest",
    "RegistrarApiTransferInDomainRequest",
    "RegistrarApiUnlockDomainTransferRequest",
    "RegistrarApiUpdateContactRequest",
    "RegistrarApiUpdateDomainHostRequest",
    "RegistrarApiUpdateDomainRequest",
    "RestoreDNSZoneVersionRequest",
    "RestoreDNSZoneVersionResponse",
    "SearchAvailableDomainsResponse",
    "UpdateDNSZoneNameserversRequest",
    "UpdateDNSZoneNameserversResponse",
    "UpdateDNSZoneRecordsRequest",
    "UpdateDNSZoneRecordsResponse",
    "UpdateDNSZoneRequest",
    "DomainV2Beta1API",
    "DomainV2Beta1RegistrarAPI",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from .types import (
    ConnectivityDiagnosticActionType,
    OS,
    ServerPrivateNetwork,
    ServerTypeCPU,
    ServerTypeDisk,
    ServerTypeGPU,
    ServerTypeMemory,
    ServerTypeNetwork,
    ServerType,
    Commitment,
    Server,
    ConnectivityDiagnosticServerHealth,
    ConnectivityDiagnostic,
    ListOSResponse,
    ListServerPrivateNetworksResponse,
    ListServerTypesResponse,
    ListServersResponse,
    SetServerPrivateNetworksResponse,
    StartConnectivityDiagnosticResponse,
    CreateServerRequest,
    PrivateNetworkApiAddServerPrivateNetworkRequest,
    PrivateNetworkApiSetServerPrivateNetworksRequest,
    ReinstallServerRequest,
    StartConnectivityDiagnosticRequest,
    CommitmentTypeValue,
    UpdateServerRequest,
)


def unmarshal_OS(data: Any) -> OS:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'OS' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("label", None)
    if field is not None:
        args["label"] = field

    field = data.get("image_url", None)
    if field is not None:
        args["image_url"] = field

    field = data.get("family", None)
    if field is not None:
        args["family"] = field

    field = data.get("is_beta", None)
    if field is not None:
        args["is_beta"] = field

    field = data.get("version", None)
    if field is not None:
        args["version"] = field

    field = data.get("xcode_version", None)
    if field is not None:
        args["xcode_version"] = field

    field = data.get("compatible_server_types", None)
    if field is not None:
        args["compatible_server_types"] = field

    return OS(**args)


def unmarshal_ServerPrivateNetwork(data: Any) -> ServerPrivateNetwork:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerPrivateNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("server_id", None)
    if field is not None:
        args["server_id"] = field

    field = data.get("private_network_id", None)
    if field is not None:
        args["private_network_id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("ipam_ip_ids", None)
    if field is not None:
        args["ipam_ip_ids"] = field

    field = data.get("vlan", None)
    if field is not None:
        args["vlan"] = field
    else:
        args["vlan"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return ServerPrivateNetwork(**args)


def unmarshal_ServerTypeCPU(data: Any) -> ServerTypeCPU:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerTypeCPU' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("core_count", None)
    if field is not None:
        args["core_count"] = field

    field = data.get("frequency", None)
    if field is not None:
        args["frequency"] = field

    return ServerTypeCPU(**args)


def unmarshal_ServerTypeDisk(data: Any) -> ServerTypeDisk:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerTypeDisk' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("capacity", None)
    if field is not None:
        args["capacity"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    return ServerTypeDisk(**args)


def unmarshal_ServerTypeGPU(data: Any) -> ServerTypeGPU:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerTypeGPU' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("count", None)
    if field is not None:
        args["count"] = field

    return ServerTypeGPU(**args)


def unmarshal_ServerTypeMemory(data: Any) -> ServerTypeMemory:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerTypeMemory' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("capacity", None)
    if field is not None:
        args["capacity"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    return ServerTypeMemory(**args)


def unmarshal_ServerTypeNetwork(data: Any) -> ServerTypeNetwork:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerTypeNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("public_bandwidth_bps", None)
    if field is not None:
        args["public_bandwidth_bps"] = field

    return ServerTypeNetwork(**args)


def unmarshal_ServerType(data: Any) -> ServerType:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ServerType' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("stock", None)
    if field is not None:
        args["stock"] = field

    field = data.get("cpu", None)
    if field is not None:
        args["cpu"] = unmarshal_ServerTypeCPU(field)
    else:
        args["cpu"] = None

    field = data.get("disk", None)
    if field is not None:
        args["disk"] = unmarshal_ServerTypeDisk(field)
    else:
        args["disk"] = None

    field = data.get("memory", None)
    if field is not None:
        args["memory"] = unmarshal_ServerTypeMemory(field)
    else:
        args["memory"] = None

    field = data.get("minimum_lease_duration", None)
    if field is not None:
        args["minimum_lease_duration"] = field
    else:
        args["minimum_lease_duration"] = None

    field = data.get("gpu", None)
    if field is not None:
        args["gpu"] = unmarshal_ServerTypeGPU(field)
    else:
        args["gpu"] = None

    field = data.get("network", None)
    if field is not None:
        args["network"] = unmarshal_ServerTypeNetwork(field)
    else:
        args["network"] = None

    field = data.get("default_os", None)
    if field is not None:
        args["default_os"] = unmarshal_OS(field)
    else:
        args["default_os"] = None

    return ServerType(**args)


def unmarshal_Commitment(data: Any) -> Commitment:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Commitment' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("cancelled", None)
    if field is not None:
        args["cancelled"] = field

    return Commitment(**args)


def unmarshal_Server(data: Any) -> Server:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Server' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("ip", None)
    if field is not None:
        args["ip"] = field

    field = data.get("vnc_url", None)
    if field is not None:
        args["vnc_url"] = field

    field = data.get("ssh_username", None)
    if field is not None:
        args["ssh_username"] = field

    field = data.get("sudo_password", None)
    if field is not None:
        args["sudo_password"] = field

    field = data.get("vnc_port", None)
    if field is not None:
        args["vnc_port"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("os", None)
    if field is not None:
        args["os"] = unmarshal_OS(field)
    else:
        args["os"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("deletable_at", None)
    if field is not None:
        args["deletable_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["deletable_at"] = None

    field = data.get("deletion_scheduled", None)
    if field is not None:
        args["deletion_scheduled"] = field

    field = data.get("zone", None)
    if field is not None:
        args["zone"] = field

    field = data.get("delivered", None)
    if field is not None:
        args["delivered"] = field

    field = data.get("vpc_status", None)
    if field is not None:
        args["vpc_status"] = field

    field = data.get("commitment", None)
    if field is not None:
        args["commitment"] = unmarshal_Commitment(field)
    else:
        args["commitment"] = None

    return Server(**args)


def unmarshal_ConnectivityDiagnosticServerHealth(
    data: Any,
) -> ConnectivityDiagnosticServerHealth:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ConnectivityDiagnosticServerHealth' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("is_server_alive", None)
    if field is not None:
        args["is_server_alive"] = field

    field = data.get("is_agent_alive", None)
    if field is not None:
        args["is_agent_alive"] = field

    field = data.get("is_mdm_alive", None)
    if field is not None:
        args["is_mdm_alive"] = field

    field = data.get("is_ssh_port_up", None)
    if field is not None:
        args["is_ssh_port_up"] = field

    field = data.get("is_vnc_port_up", None)
    if field is not None:
        args["is_vnc_port_up"] = field

    field = data.get("last_checkin_date", None)
    if field is not None:
        args["last_checkin_date"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_checkin_date"] = None

    return ConnectivityDiagnosticServerHealth(**args)


def unmarshal_ConnectivityDiagnostic(data: Any) -> ConnectivityDiagnostic:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ConnectivityDiagnostic' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("is_healthy", None)
    if field is not None:
        args["is_healthy"] = field

    field = data.get("supported_actions", None)
    if field is not None:
        args["supported_actions"] = (
            [ConnectivityDiagnosticActionType(v) for v in field]
            if field is not None
            else None
        )

    field = data.get("error_message", None)
    if field is not None:
        args["error_message"] = field

    field = data.get("health_details", None)
    if field is not None:
        args["health_details"] = unmarshal_ConnectivityDiagnosticServerHealth(field)
    else:
        args["health_details"] = None

    return ConnectivityDiagnostic(**args)


def unmarshal_ListOSResponse(data: Any) -> ListOSResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListOSResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("os", None)
    if field is not None:
        args["os"] = [unmarshal_OS(v) for v in field] if field is not None else None

    return ListOSResponse(**args)


def unmarshal_ListServerPrivateNetworksResponse(
    data: Any,
) -> ListServerPrivateNetworksResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListServerPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("server_private_networks", None)
    if field is not None:
        args["server_private_networks"] = (
            [unmarshal_ServerPrivateNetwork(v) for v in field]
            if field is not None
            else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListServerPrivateNetworksResponse(**args)


def unmarshal_ListServerTypesResponse(data: Any) -> ListServerTypesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListServerTypesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("server_types", None)
    if field is not None:
        args["server_types"] = (
            [unmarshal_ServerType(v) for v in field] if field is not None else None
        )

    return ListServerTypesResponse(**args)


def unmarshal_ListServersResponse(data: Any) -> ListServersResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListServersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("servers", None)
    if field is not None:
        args["servers"] = (
            [unmarshal_Server(v) for v in field] if field is not None else None
        )

    return ListServersResponse(**args)


def unmarshal_SetServerPrivateNetworksResponse(
    data: Any,
) -> SetServerPrivateNetworksResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'SetServerPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("server_private_networks", None)
    if field is not None:
        args["server_private_networks"] = (
            [unmarshal_ServerPrivateNetwork(v) for v in field]
            if field is not None
            else None
        )

    return SetServerPrivateNetworksResponse(**args)


def unmarshal_StartConnectivityDiagnosticResponse(
    data: Any,
) -> StartConnectivityDiagnosticResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'StartConnectivityDiagnosticResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("diagnostic_id", None)
    if field is not None:
        args["diagnostic_id"] = field

    return StartConnectivityDiagnosticResponse(**args)


def marshal_CreateServerRequest(
    request: CreateServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.type_ is not None:
        output["type"] = request.type_

    if request.enable_vpc is not None:
        output["enable_vpc"] = request.enable_vpc

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.os_id is not None:
        output["os_id"] = request.os_id

    if request.commitment_type is not None:
        output["commitment_type"] = str(request.commitment_type)

    return output


def marshal_PrivateNetworkApiAddServerPrivateNetworkRequest(
    request: PrivateNetworkApiAddServerPrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.private_network_id is not None:
        output["private_network_id"] = request.private_network_id

    if request.ipam_ip_ids is not None:
        output["ipam_ip_ids"] = request.ipam_ip_ids

    return output


def marshal_PrivateNetworkApiSetServerPrivateNetworksRequest(
    request: PrivateNetworkApiSetServerPrivateNetworksRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.per_private_network_ipam_ip_ids is not None:
        output["per_private_network_ipam_ip_ids"] = {
            key: value for key, value in request.per_private_network_ipam_ip_ids.items()
        }

    return output


def marshal_ReinstallServerRequest(
    request: ReinstallServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.os_id is not None:
        output["os_id"] = request.os_id

    return output


def marshal_StartConnectivityDiagnosticRequest(
    request: StartConnectivityDiagnosticRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_id is not None:
        output["server_id"] = request.server_id

    return output


def marshal_CommitmentTypeValue(
    request: CommitmentTypeValue,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.commitment_type is not None:
        output["commitment_type"] = str(request.commitment_type)

    return output


def marshal_UpdateServerRequest(
    request: UpdateServerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.schedule_deletion is not None:
        output["schedule_deletion"] = request.schedule_deletion

    if request.enable_vpc is not None:
        output["enable_vpc"] = request.enable_vpc

    if request.commitment_type is not None:
        output["commitment_type"] = marshal_CommitmentTypeValue(
            request.commitment_type, defaults
        )

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Zone as ScwZone,
)
from scaleway_core.utils import (
    WaitForOptions,
    random_name,
    validate_path_param,
    fetch_all_pages,
    wait_for_resource,
)
from .types import (
    ListSnapshotsRequestOrderBy,
    ListVolumesRequestOrderBy,
    CreateSnapshotRequest,
    CreateVolumeRequest,
    CreateVolumeRequestFromEmpty,
    CreateVolumeRequestFromSnapshot,
    ExportSnapshotToObjectStorageRequest,
    ImportSnapshotFromObjectStorageRequest,
    ListSnapshotsResponse,
    ListVolumeTypesResponse,
    ListVolumesResponse,
    Snapshot,
    UpdateSnapshotRequest,
    UpdateVolumeRequest,
    Volume,
    VolumeType,
)
from .content import (
    SNAPSHOT_TRANSIENT_STATUSES,
    VOLUME_TRANSIENT_STATUSES,
)
from .marshalling import (
    unmarshal_Snapshot,
    unmarshal_Volume,
    unmarshal_ListSnapshotsResponse,
    unmarshal_ListVolumeTypesResponse,
    unmarshal_ListVolumesResponse,
    marshal_CreateSnapshotRequest,
    marshal_CreateVolumeRequest,
    marshal_ExportSnapshotToObjectStorageRequest,
    marshal_ImportSnapshotFromObjectStorageRequest,
    marshal_UpdateSnapshotRequest,
    marshal_UpdateVolumeRequest,
)


class BlockV1API(API):
    """
    This API allows you to manage your Block Storage volumes.
    """

    def list_volume_types(
        self,
        *,
        zone: Optional[ScwZone] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListVolumeTypesResponse:
        """
        List volume types.
        List all available volume types in a specified zone. The volume types listed are ordered by name in ascending order.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :return: :class:`ListVolumeTypesResponse <ListVolumeTypesResponse>`

        Usage:
        ::

            result = api.list_volume_types()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/block/v1/zones/{param_zone}/volume-types",
            params={
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListVolumeTypesResponse(res.json())

    def list_volume_types_all(
        self,
        *,
        zone: Optional[ScwZone] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[VolumeType]:
        """
        List volume types.
        List all available volume types in a specified zone. The volume types listed are ordered by name in ascending order.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :return: :class:`List[VolumeType] <List[VolumeType]>`

        Usage:
        ::

            result = api.list_volume_types_all()
        """

        return fetch_all_pages(
            type=ListVolumeTypesResponse,
            key="volume_types",
            fetcher=self.list_volume_types,
            args={
                "zone": zone,
                "page": page,
                "page_size": page_size,
            },
        )

    def list_volumes(
        self,
        *,
        zone: Optional[ScwZone] = None,
        order_by: Optional[ListVolumesRequestOrderBy] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        product_resource_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> ListVolumesResponse:
        """
        List volumes.
        List all existing volumes in a specified zone. By default, the volumes listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Criteria to use when ordering the list.
        :param project_id: Filter by Project ID.
        :param organization_id: Filter by Organization ID.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :param name: Filter the return volumes by their names.
        :param product_resource_id: Filter by a product resource ID linked to this volume (such as an Instance ID).
        :param tags: Filter by tags. Only volumes with one or more matching tags will be returned.
        :return: :class:`ListVolumesResponse <ListVolumesResponse>`

        Usage:
        ::

            result = api.list_volumes()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/block/v1/zones/{param_zone}/volumes",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "product_resource_id": product_resource_id,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListVolumesResponse(res.json())

    def list_volumes_all(
        self,
        *,
        zone: Optional[ScwZone] = None,
        order_by: Optional[ListVolumesRequestOrderBy] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        name: Optional[str] = None,
        product_resource_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> List[Volume]:
        """
        List volumes.
        List all existing volumes in a specified zone. By default, the volumes listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Criteria to use when ordering the list.
        :param project_id: Filter by Project ID.
        :param organization_id: Filter by Organization ID.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :param name: Filter the return volumes by their names.
        :param product_resource_id: Filter by a product resource ID linked to this volume (such as an Instance ID).
        :param tags: Filter by tags. Only volumes with one or more matching tags will be returned.
        :return: :class:`List[Volume] <List[Volume]>`

        Usage:
        ::

            result = api.list_volumes_all()
        """

        return fetch_all_pages(
            type=ListVolumesResponse,
            key="volumes",
            fetcher=self.list_volumes,
            args={
                "zone": zone,
                "order_by": order_by,
                "project_id": project_id,
                "organization_id": organization_id,
                "page": page,
                "page_size": page_size,
                "name": name,
                "product_resource_id": product_resource_id,
                "tags": tags,
            },
        )

    def create_volume(
        self,
        *,
        zone: Optional[ScwZone] = None,
        name: Optional[str] = None,
        perf_iops: Optional[int] = None,
        project_id: Optional[str] = None,
        from_empty: Optional[CreateVolumeRequestFromEmpty] = None,
        from_snapshot: Optional[CreateVolumeRequestFromSnapshot] = None,
        tags: Optional[List[str]] = None,
    ) -> Volume:
        """
        Create a volume.
        To create a new volume from scratch, you must specify `from_empty` and the `size`.
        To create a volume from an existing snapshot, specify `from_snapshot` and the `snapshot_id` in the request payload instead, size is optional and can be specified if you need to extend the original size. The volume will take on the same volume class and underlying IOPS limitations as the original snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param name: Name of the volume.
        :param perf_iops: The maximum IO/s expected, according to the different options available in stock (`5000 | 15000`).
        One-Of ('requirements'): at most one of 'perf_iops' could be set.
        :param project_id: UUID of the project the volume belongs to.
        :param from_empty: Specify the size of the new volume if creating a new one from scratch.
        One-Of ('from'): at most one of 'from_empty', 'from_snapshot' could be set.
        :param from_snapshot: Specify the snapshot ID of the original snapshot.
        One-Of ('from'): at most one of 'from_empty', 'from_snapshot' could be set.
        :param tags: List of tags assigned to the volume.
        :return: :class:`Volume <Volume>`

        Usage:
        ::

            result = api.create_volume()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "POST",
            f"/block/v1/zones/{param_zone}/volumes",
            body=marshal_CreateVolumeRequest(
                CreateVolumeRequest(
                    zone=zone,
                    name=name or random_name(prefix="vol"),
                    project_id=project_id,
                    tags=tags,
                    from_empty=from_empty,
                    from_snapshot=from_snapshot,
                    perf_iops=perf_iops,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Volume(res.json())

    def get_volume(
        self,
        *,
        volume_id: str,
        zone: Optional[ScwZone] = None,
    ) -> Volume:
        """
        Get a volume.
        Retrieve technical information about a specific volume. Details such as size, type, and status are returned in the response.
        :param volume_id: UUID of the volume.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`Volume <Volume>`

        Usage:
        ::

            result = api.get_volume(
                volume_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_volume_id = validate_path_param("volume_id", volume_id)

        res = self._request(
            "GET",
            f"/block/v1/zones/{param_zone}/volumes/{param_volume_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Volume(res.json())

    def wait_for_volume(
        self,
        *,
        volume_id: str,
        zone: Optional[ScwZone] = None,
        options: Optional[WaitForOptions[Volume, bool]] = None,
    ) -> Volume:
        """
        Get a volume.
        Retrieve technical information about a specific volume. Details such as size, type, and status are returned in the response.
        :param volume_id: UUID of the volume.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`Volume <Volume>`

        Usage:
        ::

            result = api.get_volume(
                volume_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in VOLUME_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_volume,
            options=options,
            args={
                "volume_id": volume_id,
                "zone": zone,
            },
        )

    def delete_volume(
        self,
        *,
        volume_id: str,
        zone: Optional[ScwZone] = None,
    ) -> None:
        """
        Delete a detached volume.
        You must specify the `volume_id` of the volume you want to delete. The volume must not be in the `in_use` status.
        :param volume_id: UUID of the volume.
        :param zone: Zone to target. If none is passed will use default zone from the config.

        Usage:
        ::

            result = api.delete_volume(
                volume_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_volume_id = validate_path_param("volume_id", volume_id)

        res = self._request(
            "DELETE",
            f"/block/v1/zones/{param_zone}/volumes/{param_volume_id}",
        )

        self._throw_on_error(res)

    def update_volume(
        self,
        *,
        volume_id: str,
        zone: Optional[ScwZone] = None,
        name: Optional[str] = None,
        size: Optional[int] = None,
        tags: Optional[List[str]] = None,
        perf_iops: Optional[int] = None,
    ) -> Volume:
        """
        Update a volume.
        Update the technical details of a volume, such as its name, tags, or its new size and `volume_type` (within the same Block Storage class).
        You can only resize a volume to a larger size. It is currently not possible to change your Block Storage Class.
        :param volume_id: UUID of the volume.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param name: When defined, is the new name of the volume.
        :param size: Size in bytes of the volume, with a granularity of 1 GB (10^9 bytes).
        Must be compliant with the minimum (1GB) and maximum (10TB) allowed size.
        :param tags: List of tags assigned to the volume.
        :param perf_iops: The selected value must be available for the volume's current storage class.
        :return: :class:`Volume <Volume>`

        Usage:
        ::

            result = api.update_volume(
                volume_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_volume_id = validate_path_param("volume_id", volume_id)

        res = self._request(
            "PATCH",
            f"/block/v1/zones/{param_zone}/volumes/{param_volume_id}",
            body=marshal_UpdateVolumeRequest(
                UpdateVolumeRequest(
                    volume_id=volume_id,
                    zone=zone,
                    name=name,
                    size=size,
                    tags=tags,
                    perf_iops=perf_iops,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Volume(res.json())

    def list_snapshots(
        self,
        *,
        zone: Optional[ScwZone] = None,
        order_by: Optional[ListSnapshotsRequestOrderBy] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        volume_id: Optional[str] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> ListSnapshotsResponse:
        """
        List all snapshots.
        List all available snapshots in a specified zone. By default, the snapshots listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Criteria to use when ordering the list.
        :param project_id: Filter by Project ID.
        :param organization_id: Filter by Organization ID.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :param volume_id: Filter snapshots by the ID of the original volume.
        :param name: Filter snapshots by their names.
        :param tags: Filter by tags. Only snapshots with one or more matching tags will be returned.
        :return: :class:`ListSnapshotsResponse <ListSnapshotsResponse>`

        Usage:
        ::

            result = api.list_snapshots()
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "GET",
            f"/block/v1/zones/{param_zone}/snapshots",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
                "volume_id": volume_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListSnapshotsResponse(res.json())

    def list_snapshots_all(
        self,
        *,
        zone: Optional[ScwZone] = None,
        order_by: Optional[ListSnapshotsRequestOrderBy] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        volume_id: Optional[str] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> List[Snapshot]:
        """
        List all snapshots.
        List all available snapshots in a specified zone. By default, the snapshots listed are ordered by creation date in ascending order. This can be modified via the `order_by` field.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param order_by: Criteria to use when ordering the list.
        :param project_id: Filter by Project ID.
        :param organization_id: Filter by Organization ID.
        :param page: Page number.
        :param page_size: Page size, defines how many entries are returned in one page, must be lower or equal to 100.
        :param volume_id: Filter snapshots by the ID of the original volume.
        :param name: Filter snapshots by their names.
        :param tags: Filter by tags. Only snapshots with one or more matching tags will be returned.
        :return: :class:`List[Snapshot] <List[Snapshot]>`

        Usage:
        ::

            result = api.list_snapshots_all()
        """

        return fetch_all_pages(
            type=ListSnapshotsResponse,
            key="snapshots",
            fetcher=self.list_snapshots,
            args={
                "zone": zone,
                "order_by": order_by,
                "project_id": project_id,
                "organization_id": organization_id,
                "page": page,
                "page_size": page_size,
                "volume_id": volume_id,
                "name": name,
                "tags": tags,
            },
        )

    def get_snapshot(
        self,
        *,
        snapshot_id: str,
        zone: Optional[ScwZone] = None,
    ) -> Snapshot:
        """
        Get a snapshot.
        Retrieve technical information about a specific snapshot. Details such as size, volume type, and status are returned in the response.
        :param snapshot_id: UUID of the snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.get_snapshot(
                snapshot_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "GET",
            f"/block/v1/zones/{param_zone}/snapshots/{param_snapshot_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def wait_for_snapshot(
        self,
        *,
        snapshot_id: str,
        zone: Optional[ScwZone] = None,
        options: Optional[WaitForOptions[Snapshot, bool]] = None,
    ) -> Snapshot:
        """
        Get a snapshot.
        Retrieve technical information about a specific snapshot. Details such as size, volume type, and status are returned in the response.
        :param snapshot_id: UUID of the snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.get_snapshot(
                snapshot_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in SNAPSHOT_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_snapshot,
            options=options,
            args={
                "snapshot_id": snapshot_id,
                "zone": zone,
            },
        )

    def create_snapshot(
        self,
        *,
        volume_id: str,
        zone: Optional[ScwZone] = None,
        name: Optional[str] = None,
        project_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Snapshot:
        """
        Create a snapshot of a volume.
        To create a snapshot, the volume must be in the `in_use` or the `available` status.
        If your volume is in a transient state, you need to wait until the end of the current operation.
        :param volume_id: UUID of the volume to snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param name: Name of the snapshot.
        :param project_id: UUID of the project to which the volume and the snapshot belong.
        :param tags: List of tags assigned to the snapshot.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.create_snapshot(
                volume_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "POST",
            f"/block/v1/zones/{param_zone}/snapshots",
            body=marshal_CreateSnapshotRequest(
                CreateSnapshotRequest(
                    volume_id=volume_id,
                    zone=zone,
                    name=name or random_name(prefix="snp"),
                    project_id=project_id,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def import_snapshot_from_object_storage(
        self,
        *,
        bucket: str,
        key: str,
        name: str,
        zone: Optional[ScwZone] = None,
        project_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        size: Optional[int] = None,
    ) -> Snapshot:
        """
        Import a snapshot from a Scaleway Object Storage bucket.
        The bucket must contain a QCOW2 image.
        The bucket can be imported into any Availability Zone as long as it is in the same region as the bucket.
        :param bucket: Scaleway Object Storage bucket where the object is stored.
        :param key: The object key inside the given bucket.
        :param name: Name of the snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param project_id: UUID of the Project to which the volume and the snapshot belong.
        :param tags: List of tags assigned to the snapshot.
        :param size: Size of the snapshot.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.import_snapshot_from_object_storage(
                bucket="example",
                key="example",
                name="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)

        res = self._request(
            "POST",
            f"/block/v1/zones/{param_zone}/snapshots/import-from-object-storage",
            body=marshal_ImportSnapshotFromObjectStorageRequest(
                ImportSnapshotFromObjectStorageRequest(
                    bucket=bucket,
                    key=key,
                    name=name,
                    zone=zone,
                    project_id=project_id,
                    tags=tags,
                    size=size,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def export_snapshot_to_object_storage(
        self,
        *,
        snapshot_id: str,
        bucket: str,
        key: str,
        zone: Optional[ScwZone] = None,
    ) -> Snapshot:
        """
        Export a snapshot to a Scaleway Object Storage bucket.
        The snapshot is exported in QCOW2 format.
        The snapshot must not be in transient state.
        :param snapshot_id: UUID of the snapshot.
        :param bucket: Scaleway Object Storage bucket where the object is stored.
        :param key: The object key inside the given bucket.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.export_snapshot_to_object_storage(
                snapshot_id="example",
                bucket="example",
                key="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "POST",
            f"/block/v1/zones/{param_zone}/snapshots/{param_snapshot_id}/export-to-object-storage",
            body=marshal_ExportSnapshotToObjectStorageRequest(
                ExportSnapshotToObjectStorageRequest(
                    snapshot_id=snapshot_id,
                    bucket=bucket,
                    key=key,
                    zone=zone,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())

    def delete_snapshot(
        self,
        *,
        snapshot_id: str,
        zone: Optional[ScwZone] = None,
    ) -> None:
        """
        Delete a snapshot.
        You must specify the `snapshot_id` of the snapshot you want to delete. The snapshot must not be in use.
        :param snapshot_id: UUID of the snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.

        Usage:
        ::

            result = api.delete_snapshot(
                snapshot_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "DELETE",
            f"/block/v1/zones/{param_zone}/snapshots/{param_snapshot_id}",
        )

        self._throw_on_error(res)

    def update_snapshot(
        self,
        *,
        snapshot_id: str,
        zone: Optional[ScwZone] = None,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Snapshot:
        """
        Update a snapshot.
        Update the name or tags of the snapshot.
        :param snapshot_id: UUID of the snapshot.
        :param zone: Zone to target. If none is passed will use default zone from the config.
        :param name: When defined, is the name of the snapshot.
        :param tags: List of tags assigned to the snapshot.
        :return: :class:`Snapshot <Snapshot>`

        Usage:
        ::

            result = api.update_snapshot(
                snapshot_id="example",
            )
        """

        param_zone = validate_path_param("zone", zone or self.client.default_zone)
        param_snapshot_id = validate_path_param("snapshot_id", snapshot_id)

        res = self._request(
            "PATCH",
            f"/block/v1/zones/{param_zone}/snapshots/{param_snapshot_id}",
            body=marshal_UpdateSnapshotRequest(
                UpdateSnapshotRequest(
                    snapshot_id=snapshot_id,
                    zone=zone,
                    name=name,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Snapshot(res.json())
